#!/usr/bin/python

# This code is in the public domain.

# do an all-VO obscore search for spectra around a list of points.


import sys

from astropy import table
import pyvo

import vohelper


def iter_obscore_urls():
  """yields access URLs for obscore services.
  """
  # use raw RegTAP until pyVO registry is up to the task
  result = pyvo.dal.TAPService("http://reg.g-vo.org/tap"
    ).run_sync("""
    SELECT DISTINCT access_url AS url
    FROM rr.interface
    NATURAL JOIN rr.capability
    NATURAL JOIN rr.res_detail
    WHERE standard_id='ivo://ivoa.net/std/tap'
       AND intf_type='vs:paramhttp'
       AND detail_xpath='/capability/dataModel/@ivo-id'
       AND 1=ivo_nocasematch(detail_value, 
         'ivo://ivoa.net/std/obscore%')""")

  for url in result["url"]:
    yield url


def get_spectra_for_table(obscore_url, pois, radius, samplesize):
  """yields pairs of (dataset name, access_url) for spectra within radius
  degrees of points in pois for and obscore service.
  """
  ra_column_name = vohelper.get_name_for_ucd(
    "pos.eq.ra;meta.main", pois)
  dec_column_name = vohelper.get_name_for_ucd(
    "pos.eq.dec;meta.main", pois)

  # the rstrip in the next line is a workaround for a botched registration of
  # VAO
  try:
    svc = pyvo.dal.TAPService(obscore_url.rstrip("?"))
    if not svc.upload_methods:
      # service doesn't support upload, can't use it
      return
  except Exception, msg:
    # service unusable, skip it (but log, as whenever you catch all exceptions)
    print("broken: {}".format(msg))
    return
  except KeyboardInterrupt:
		# let users interrupt hanging/slow services
		return

  # you'd normally really match
  # CONTAINS(POINT(up.ra, up.dec), s_region); however, we need to fudge here
  # since there's still too little data in obscore.
  result = vohelper.run_sync_resilient(svc,
    """SELECT TOP {samplesize} oc.obs_publisher_did, oc.access_url
        FROM ivoa.obscore AS oc
        JOIN TAP_UPLOAD.pois AS mine
        ON 1=CONTAINS(
          POINT('ICRS', oc.s_ra, oc.s_dec),
          CIRCLE('ICRS', 
            mine.{ra_column_name}, 
            mine.{dec_column_name}, 
            {radius}))
        WHERE oc.dataproduct_type='spectrum'
        """.format(**locals()),
        # add more constraints (spectral region, resolution... here)
        uploads = {"pois": pois})

  if result is None:
    return

  for row in result.table:
    yield unicode(row[0]), unicode(row[1])


def main():
  args = sys.argv+["selected_positions.vot", "1000", "2"][len(sys.argv)-1:]
  
  with open(args[1]) as f:
    pois = table.Table.read(f)
  radius = float(args[2])/3600
  n_samp = int(args[3])

  with vohelper.SAMP_conn() as conn:
    try:
      target_id = vohelper.find_client(conn, "splat")
    except KeyError:
      sys.exit("Start Splat and try again")

    for access_url in iter_obscore_urls():
      sys.stdout.write("Querying {} ...".format(access_url))
      sys.stdout.flush()

      specs = list(get_spectra_for_table(access_url, pois, radius, n_samp))
      sys.stdout.write(" done. ({})\n".format(len(specs)))

      for ds_name, access_url in specs:
        print("Opening {}...".format(access_url))
        try:
          vohelper.send_spectrum_to(conn, target_id, access_url, ds_name)
        except vohelper.SAMPProxyError:
          print("  (Failed)")


if __name__=="__main__":
  main()
